/* $Id: confiner.c,v 1.3 2002/02/09 20:46:46 queinnec Exp $ 
 Copyright (C) 2002 by <Christian.Queinnec@lip6.fr>
 GPL >= 2.
*/

#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <unistd.h>
#include <assert.h>
#include <sys/types.h>
#include <sys/wait.h>

#define _GNU_SOURCE
#include <getopt.h>
extern char *optarg;
extern int optind, opterr, optopt;

char *usage = ""
"confine [restrictions...] -- command arguments ... \n"
"Copyright (C) 2002 by <Christian.Queinnec@lip6.fr> $Revision: 1.3 $\n"
"GPL >= 2\n"
"\n"
"Restrictions may be: \n"
"  --cpu=s     restrict cpu to n seconds,\n"
"  --fsize=n   restricts the number of output bytes\n"
"  --data=n    restricts data segment to n bytes\n"
"  --core=n    restricts core to n bytes\n"
"  --nproc=n   restricts the number of process to n\n"
"  --nofile=n  restricts the number of open files to n\n";

#define CONFINER_ERROR_PREFIX   "#<$!@(ERROR)+*&> "
#define CONFINER_WARNING_PREFIX "#<$!@(WARNING)+*&> "
#define KILO (1000)
#define MEGA (KILO * KILO)

/* 
   Lower if necessary a given resource.

   QUESTION: le man dit qu'il y a des limites hard et soft. Ici il
   semble que l'on ne touche qu'aux limites soft qui sont modifiables:
   on ne peut donc pas les imposer.

 */

void
lower (int resource, rlim_t value)
{
     struct rlimit rlim;
     int code;
     
     code = getrlimit(resource, &rlim);
     if ( code != 0 ) {
          fprintf(stderr, CONFINER_ERROR_PREFIX "Cannot get limit %d\n", 
                  resource);
          exit(1);
     };
     if ( value < rlim.rlim_cur ) {
          rlim.rlim_cur = value;
     };
     code = setrlimit(resource, &rlim);
     if ( code != 0 ) {
          fprintf(stderr, CONFINER_ERROR_PREFIX "Cannot set limit %d\n", 
                  resource);
          exit(1);
     };
}

int 
main (int  argc,
      char *argv[])
{
     int code = 0;
     int longindex = 1;
     pid_t pid, otherpid;

     int cpu    = 30;         /* seconds */
     int fsize  =  50 * KILO;
     int data   = 100 * MEGA;
     int stack  =  10 * MEGA;
     int core   =   1 * KILO;
     int rss    =  10 * MEGA;
     int nproc  = 100;
     int nofile = 100;

     /* Should take the same letters as ulimit.            FUTURE */
     struct option longopts[10] = {
          {"cpu",    required_argument, NULL, 't'},
          {"fsize",  required_argument, NULL, 'o'},
          {"data",   required_argument, NULL, 'a'},
          {"stack",  required_argument, NULL, 's'},
          {"core",   required_argument, NULL, 'd'},
          {"rss",    required_argument, NULL, 'r'},
          {"nproc",  required_argument, NULL, 'p'},
          {"nofile", required_argument, NULL, 'f'},
          {"help",   no_argument,       NULL, 'h'},
          {NULL,     0,                 NULL, 0}
     };

     /* Decode parameters */
     while ( 1 ) {
          code = getopt_long(argc, argv, 
                             "t:o:a:s:d:r:p:f:h",
                             longopts, &longindex );
          if ( code == -1 ) {
               break;
          };
          switch (code) {
               case ':' : { 
                    fprintf(stderr, CONFINER_ERROR_PREFIX 
                            "Missing option parameter for %s\n",
                            longopts[longindex].name );
                    exit(5);
               }
               case '?' : {
                    fprintf(stderr, CONFINER_ERROR_PREFIX 
                            "Ambiguous/missing option for %s\n",
                            longopts[longindex].name );
                    exit(5);
               }
               case 't': {
                    cpu = (rlim_t) strtoul(optarg, NULL, 10);
                    break;
               }
               case 'o': {
                    fsize = (rlim_t) strtoul(optarg, NULL, 10);
                    break;
               }
               case 'a': {
                    data = (rlim_t) strtoul(optarg, NULL, 10);
                    break;
               }
               case 's': {
                    stack = (rlim_t) strtoul(optarg, NULL, 10);
                    break;
               }
               case 'd': {
                    core = (rlim_t) strtoul(optarg, NULL, 10);
                    break;
               }
               case 'r': {
                    rss = (rlim_t) strtoul(optarg, NULL, 10);
                    break;
               }
               case 'p': {
                    nproc = (rlim_t) strtoul(optarg, NULL, 10);
                    break;
               }
               case 'f': {
                    nofile = (rlim_t) strtoul(optarg, NULL, 10);
                    break;
               }
               case 'h': {
                    fprintf(stderr, usage);
                    return EXIT_SUCCESS;
               }
               default : {
                    fprintf(stderr, CONFINER_ERROR_PREFIX "Impossible");
                    exit(3);
               }
          }
     }

     /* Set limits */
     lower(RLIMIT_CPU,    cpu);
     lower(RLIMIT_FSIZE,  fsize);
     lower(RLIMIT_DATA,   data);
     /* lower(RLIMIT_STACK,  stack); */
     lower(RLIMIT_CORE,   core);
     /* lower(RLIMIT_RSS,    rss); */
     lower(RLIMIT_NPROC,  nproc);
     lower(RLIMIT_NOFILE, nofile);

     /* Execute the confined command */
     pid = fork();
     if ( pid > 0 ) {
          /* The father waits for the son (and siblings) */
          while ( 1 ) {
               otherpid = wait(&code);
               if ( otherpid == -1 ) {
                    perror(CONFINER_ERROR_PREFIX "wait() problem");
                    return EXIT_FAILURE;
               } else if ( otherpid == pid ) {
                    if ( WIFEXITED(code) != 0 ) {
                         /* Son exited normally */
                         return WEXITSTATUS(code);
                    } else {
                         /* Make sure to return an erroneous code: */
                         code = WEXITSTATUS(code);
                         fprintf(stderr, CONFINER_ERROR_PREFIX 
                                 "Abnormal exit, code: %d\n", code);
                         code = (code == 0) ? 1 : code;
                         return code;
                    }
               } else if ( WIFSIGNALED(code) ) {
                    /* Son exited abnormally */
                    fprintf(stderr, CONFINER_ERROR_PREFIX 
                            "Uncaught signal %d\n", WTERMSIG(code));
               }
          }
          /* UNREACHED */
          return EXIT_SUCCESS;
     } else {
          code = execvp(argv[optind], &(argv[optind]));
          perror(CONFINER_ERROR_PREFIX "Could not exec the command:");
          return EXIT_FAILURE;
     }
}

/* end of confiner.c */
