#! /usr/bin/perl
# $Id: extract.pl,v 1.15 2002/11/25 21:27:38 queinnec Exp $
# Copyright (C) 2002 by <Christian.Queinnec@lip6.fr>

# This script takes a number of argument, extracts marks from a file
# and convert them into SQL requests.

# Three mandatory arguments:
#       the tgz to mark,
#             The tgz should have the following name:
#                    p6lipXXX_blabla_*
#                 or p6lipXXX_p6lipYYY_*
#                 or blabla_p6lipXXX_*
#       the uuid used to identify the student's copy,
#       the name of the CFS session (something as CFS2002jun30am),
# The rest is the file to scan to extract marks from.

my $tgz = shift;

if ( $tgz =~ /^-(h|-help)$/ ) {
    print "
-- Usage: $0 file.tar.gz uuid session logfile
-- Read logfile to generate SQL insert statements.
-- See create.sql for the SQL tables.
";
    exit(0);
};

my $uuid = shift;
my $session = shift;

# Scan the name of the tgz and find the login names of one or two students.
# DelBul changes the rules for naming files: now the name is 
#     p6lip6_blabla_001.tar.gz
# or  blabla_p6lip6_001.tar.gz
# Due to the fact that for some examinations we have both p6lip and p6mip
# logins, I support the two. See also RT_TGZ_PREFIX in run.sh

if ( $tgz =~ /[\/_](p6.ip\d+)_/ ) {
    # Extract login from deliverable name:
    my $login = $1;
    if ( $tgz =~ /^.*\/([^\/]*)$/ ) {
        # reduce tgz to its basename:
        $tgz = $1;
    };
    print "insert into :STUDENT values('$uuid', '$login', '$tgz', '$session');\n";
    if ( $tgz =~ /_(p6.ip\d+)_/ ) {
        my $otherlogin = $1;
        if ( $otherlogin ne $login ) {
            print "insert into :STUDENT values('$uuid', '$otherlogin', '$tgz', '$session');\n";
        };
    };
} else {
    die "Cannot parse $tgz";
}

# Scan the last files. Extract SQL commands only from XML tags with
# the right attribute. This is to prevent students to generate output
# that looks like these tags.

# NOTE: the name of a question should be something like *-Qnnn where
# nnn identifies the question. I often use something as 06-05-02-Q1 as
# this identifier is also used in Delivery Builder. But only the last
# part is used for the database (the part starting with Q). 

while ( <> ) {
    if ( /\<MARK_SUM +question=\"([^\"]+)\" +uuid=\"($uuid)\"\>\s*(-?[\d\.]+)\s*\<\/MARK_SUM\>/ ) {
        my $question = $1;
        my $sum = $3;
        if ( $question =~ /\-?(Q.+)$/i ) {
            $question = $1;
        };
        print "insert into :MARK(uuid, question, mark) values('$uuid', '$question', $sum);\n";
    } elsif ( /\<FACT +question=\'([^\"]+)\' +label=\'([^\']*)\' +(phrase|number)=\'([^\']*)\' +uuid=\"($uuid)\"\>/ ) {
        my ($question, $label, $key, $value) = ($1, $2, $3, $4);
        if ( $question =~ /\-?(Q.+)$/i ) {
            $question = $1;
        };
        $value =~ s/\'/\\\'/g;
        print "insert into :FACT(uuid, question, label, $key) values('$uuid', '$question', '$label', '$value');\n";
    };
}

# end of extract.pl
