#! /bin/sh
# $Id: limits.sh,v 1.3 2002/09/24 20:10:30 queinnec Exp $
# Copyright (C) 2002 by <Christian.Queinnec@lip6.fr>
# GPL version 2.

# This script sets limits at various levels.
option=$1
shift

case "$option" in 

    # These are the general limits that are set up for the whole marking
    # process. It limits what teacher's and student's code may do.
    general)
      ulimit -c      1 # 1 k (maximum size of core files created)
      ulimit -d 110000 # 110 meg (maximum size of a process's data segment)
      ulimit -f  11000 # 11 meg (maximum size of files created by the shell)
      ulimit -n    200 # (maximum number of open file descriptors)
      ulimit -s  11000 # 11 meg (maximum stack size)
      ulimit -m  11000 # 11 meg (maximum resident set size)
      ulimit -t    120 # 120 seconds (maximum amount of cpu time)
      ulimit -u    150 # maximum number of processes available to a single user
    ;;

    # These are the limits that a single teacher's test can't overpass.
    # It should be less than the general limits.
    single)
      # 1 k (maximum size of core files created):
      ulimit -c      1 
      # 100 meg (maximum size of a process's data segment):
      ulimit -d 100000
      # 10.5 meg (maximum size of files created by the shell):
      ulimit -f   10500
      # (maximum number of open file descriptors):
      ulimit -n    100 
      # 10 meg (maximum stack size):
      ulimit -s  10000
      # 10 meg (maximum resident set size):
      ulimit -m  10000
      # 100 seconds (maximum amount of cpu time):
      ulimit -t    100
      # (maximum number of processes available to a single user):
      ulimit -u    100 
    ;;

    # Limit the CPU-time for a given teacher's task.
    limited-single)
      # 100 seconds (maximum amount of cpu time):
      ulimit -t    $1
    ;;

    # These are the limits for student's code. Configurable limits
    # for -t and -f may be individually set up in the rt_confiner function.
    # It should be less than the single limits.
    student|*)
      ulimit -c      1      # core < 1 k
      ulimit -d 100000      # 100 meg data segment
      ulimit -n    100      # 100 maximum number of opened files
      ulimit -s  10000      # 10 meg stack size
      ulimit -m  10000      # 10 meg rss size
      ulimit -u    100      # 100 processes at most
      # -t and -f inherited from single.
    ;;
esac

# end of limits.sh
