#! /bin/sh
# $Id: run.sh,v 1.28 2004/01/22 17:01:16 queinnec Exp $
# Copyright (C) 2002, 2003 by <Christian.Queinnec@lip6.fr>
# GPL version 2.

# Run this as:          
#       ./run.sh [-n] /Infos/licence/2002/CFS2002jan/students/ ...
# These are the directories where are the p6lip*.tar.gz deliveries to mark.
# These directories should have absolute names. The -n option allows to
# only mark the lattest tarball from any student.

# It is also possible to mark only some p6lip*.tar.gz files as in:
#       ./run.sh /Infos/licence/2002/CFS2002jan/students/*p6lip*.gz
# NOTE: the -n option is not allowed there.

# Where is the cfs kit ? It is probably better to let the CFSKIT_DIR
# variable to be set by the caller. This directory should contain this
# run.sh script.

CFSKIT_DIR=${CFSKIT_DIR:?"La variable CFSKIT_DIR n'est pas definie!"}

# Where are the shell scripts (and among them the current one). By
# default, they stay in the src subdirectory of the CFSKIT directory):

export RT_MARK_DIR=${RT_MARK_DIR:-$CFSKIT_DIR}

# Where are the tests. This directory contains directories containing
# the tests for all the questions to mark. That is the 
#     /tests/            directory contains
#     /tests/Q1/         directory containing the scripts to mark question 1
#     /tests/Q1/00test   first test for question 1

export RT_TESTS_DIR=${RT_TESTS_DIR:-$CFSKIT_DIR/CFS2002jan30/tests}

# Where will appear the final logs that is the result of the notation.

export RT_LOG_DIR=${RT_LOG_DIR:-/tmp/logs}

# The RT_SESSION identifier identifies the examination and will be
# used to tag all the produced marks in the database.

export RT_SESSION=${RT_SESSION:-`basename $1`}

# Mark only the tar.gz from delivery builder whose name contains that
# string. May be used to restrict a big set of tgz into a smaller subset.

export RT_TGZ_PREFIX=${RT_TGZ_PREFIX:-p6\?ip}

# Use that suffix to find the tgz to mark. By default, this is tar.gz,
# it might be redefined to be tgz. The leading point separating the
# basename from the extension must be omitted.

export RT_TGZ_SUFFIX=${RT_TGZ_SUFFIX:-tar.gz}

# What is the HTTP prefix leading to the /tests/ directory. This is
# needed only when links to test scripts are to be generated in
# all.html. The student may then click to obtain the source code of
# the tests. 

export RT_HTTP_PREFIX=${RT_HTTP_PREFIX:-'.'}

# Sometimes to debug the tests, it is useful to keep the student's
# directory where the students' programs are compiled and run. Don't
# set it to true when marking lots of copies, it may take a huge
# amount of disk space. But set it to true if you intend to perform
# some global post-processing on all students' copies.

export RT_KEEP_RUN_DIR=${RT_KEEP_RUN_DIR:-false}

# Same for the temporary directory where temporary results may be stored.

export RT_KEEP_TMP_DIR=${RT_KEEP_TMP_DIR:-false}

# Extract marks from logs and record them in the database.

export RT_USE_DB=${RT_USE_DB:-true}

# Massage the XML log to form a complete HTML page.

export RT_MAKE_HTML=${RT_MAKE_HTML:-true}

# Tell the URL to the CSS to use. The URL may be complete (ie starting
# with http://...) or absolute (starting with / in which case it is
# relative to the current httpd server).

export RT_CSS=${RT_CSS:-/licence/licence.css}

# Associated to a link to the table of contents (toc) of the marked
# copy. This allows to go to the toc from the title of any question.
# The default image corresponds to a triangle pointing upward.

export RT_UP_IMAGE=${RT_UP_IMAGE:="<span class='UP'><img border=0 src='/dbufr/public/up.png' /></a>"}

# Try to mark copies concurrently otherwise they are marked
# sequentially and this may be very long. If set, the -p option will
# supersed that default.

export RT_PARALLEL=${RT_PARALLEL:=false}
export RT_STRIDE=${RT_STRIDE:=3}

# Mark only the most recent copies. If set, the -n option will
# supersede that default.

export RT_NEWEST=${RT_NEWEST:=false}

# ###################################### Check some settings

if [ -f "${CFSKIT_DIR}/run.sh" ]
then true
else echo "
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!???????????????????????????????????
La variable CFSKIT_DIR ne semble pas correcte, je ne trouve pas le
script ${CFSKIT_DIR}/run.sh
???????????????????????????????????!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
"
     exit 1
fi

# ###################################### Library
# A function to mark a single tarball. There may be at most RT_STRIDE
# copies marked concurrently.

function mark {
    if $RT_PARALLEL
    then # Mygale has an old test program that ignores parentheses:
         N=$(( $1 % $RT_STRIDE ))
         if [ 0 -eq $N ]
         then wait
         fi
         ( do_mark "$2" ) &
    else ( do_mark "$2" )
    fi
}

function do_mark {
    export RT_UUID=${RT_UUID:-`uuidgen`}
    ( 
      # If RT_UUID is already set that means that we re-use that uuid.
      # Probably to mark a tarball again. 
      if [ -d ${RT_UUID} ]
      then
         echo "Moving previous ${RT_UUID} to ${RT_UUID}-$$" 1>&2
         mv ${RT_UUID} ./${RT_UUID}-$$
      fi
      mkdir ${RT_UUID} && cd ${RT_UUID} && tar xzf $1
      # make the student be there as if at home:
      export HOME=`pwd`
      #mkdir $HOME/tmp
      #export TMPDIR=$HOME/tmp
      #export TMP=$TMPDIR
      echo "Marking $1 ..." 1>&2
      . ${RT_MARK_DIR}/limits.sh general
      ${RT_MARK_DIR}/runalltests.sh "$1"
      # Normally $RT_LOG_DIR/$RT_UUID/all.{log,*} are now generated.
     )
     # Erase the directory where computation is performed (unless
     # RT_KEEP_RUN_DIR)
     if ${RT_KEEP_RUN_DIR}
     then false
     else rm -rf ${RT_UUID}
     fi
     # Extract the facts to be inserted in the DB from the log file,
     # that is, generate the all.sql file:
     if ${RT_USE_DB}
     then perl ${RT_MARK_DIR}/extract.pl $1 ${RT_UUID} ${RT_SESSION} \
        ${RT_LOG_DIR}/${RT_UUID}/all.log > $RT_LOG_DIR/${RT_UUID}/all.sql
     fi
}

# Delivery_Builder accumulates tgz as p6lip*_001.tar.gz,
# p6lip*_002.tar.gz, etc. The -n option allows to mark only the newest
# (most recent) tarball from any series. Improvement suggested by
# <Lionel.Seinturier@lip6.fr>

while true
do
  case "$1" in
      -n) 
          RT_NEWEST=true
          shift
          echo "Only marking newest tarballs ..." 1>&2 
          ;;
      -p[0-9])
          RT_PARALLEL=true
          RT_STRIDE=0${1#-p}
          shift
          echo "Marking tarballs ${RT_STRIDE}-concurrently ..." 1>&2
          ;;
      -p)
          RT_PARALLEL=true
          export RT_STRIDE=10000
          shift
          echo "Marking tarballs very concurrently ..." 1>&2
          ;;
      *)
          break
          ;;
  esac
done

# Analyze the remaining arguments: tgz or directories containing tgz.

for thing in "$@"
do 
    if [ -d $thing ]
    then 
        echo "Marking directory $thing ... " 1>&2
        if $RT_NEWEST
        then
            RT_N=`ls $thing/*${RT_TGZ_PREFIX}*_001.${RT_TGZ_SUFFIX} | wc -l | { read n; echo $n; }`
            RT_I=0
            for tgz in $thing/*${RT_TGZ_PREFIX}*_001.${RT_TGZ_SUFFIX}
            do 
                base=`basename $tgz 001.${RT_TGZ_SUFFIX}`
                tgz=`ls -t $thing/${base}* | head -1`
                mark $RT_I $tgz 
                RT_I=$(($RT_I + 1))
                echo "*** Directory $thing, sheet $RT_I/$RT_N." 1>&2
            done
        else
            RT_N=`ls $thing/*${RT_TGZ_PREFIX}*.${RT_TGZ_SUFFIX} | wc -l | { read n; echo $n; }`
            RT_I=0
            for tgz in $thing/*${RT_TGZ_PREFIX}*.${RT_TGZ_SUFFIX} 
            do 
                mark $RT_I $tgz 
                RT_I=$(($RT_I + 1))
                echo "*** Directory $thing, sheet $RT_I/$RT_N." 1>&2
            done
        fi
    elif [ -f $thing ] 
    then 
        case "X$thing" in
            X/*/*${RT_TGZ_PREFIX}*.${RT_TGZ_SUFFIX})
                mark $RT_I $thing 
                ;;
            *)
                echo "Ignoring $thing." 1>&2
                ;;
        esac
    else
        echo "Ignoring this: $thing." 1>&2
    fi
done

# Wait for all marking processes to finish:

if $RT_PARALLEL
then wait
fi

# Perform global (that is, on all students' copies) tests if any:

if [ -x ${RT_TESTS_DIR}/post.sh ]
then
    if ${RT_KEEP_RUN_DIR}
    then 
        echo "***---*** Post processing all students' sheets ..."
        ${RT_TESTS_DIR}/post.sh
    else 
        echo "??????? No global post-processing possible: RUN_DIRS removed"
        exit 1
    fi
fi

# end of run.sh
