#! /bin/sh
# $Id: runtestlib.sh,v 1.54 2004/04/16 13:22:45 queinnec Exp $
# Copyright (C) 2002, 2003 by <Christian.Queinnec@Lip6.fr>
# GPL version 2.

# These two variables customize the Levenshtein distance:
export RT_VERBOSE=${RT_VERBOSE:-false}
export RT_CLEANUP=${RT_CLEANUP:-true}

# Library of functions for tests.
# {{{ XML utilities

# Emit an opening XML tag:

function rt_start_xml {
    echo -n "<$1>"
}

function rt_start_xml_with_id {
    echo -n "<$1 uuid=\"${RT_UUID}\">"
}

# Emit a closing XML tag:

function rt_stop_xml {
    echo "</$1>"
}

# Emit an XML comment:

function rt_comment {
    echo '<!--' "$@" '-->'
}

function rt_tagged_embed {
    echo "<$1>$2</$1>"
}

# Notify that a function is deprecated and should not be used any longer.

function rt_notify_deprecation {
    ( echo -n '!!!!!!!!!!'; echo "$@" ) 1>&2
}

# Usage: rt_warning text ...
# Notify something wrong or weird.

function rt_warning {
    rt_start_xml "span class=\"WARNING\""
    echo "[Note: "
    echo "$@"
    echo "]"
    rt_stop_xml span
}

# Usage: rt_error text ...
# Report an error:

function rt_error {
    rt_start_xml "p class=\"ERROR\""
    echo "ERREUR! "
    echo "$@"
    rt_stop_xml p
}

function rt_internal_error {
    echo "++++++++++++++++++++++++++++++++++++++++++++++++++" 1>&2
    echo "GROS GROS PROBLEME INTERNE: " "$@" 1>&2
    echo "++++++++++++++++++++++++++++++++++++++++++++++++++" 1>&2
    exit 126
}

# Usage: rt_echo text ...
# Report something:

function rt_echo {
    rt_start_xml "span class=\"ECHO\""
    echo ; echo "$@" " "
    rt_stop_xml span
}

# Sections. 

# La question courante a son numero comme nom et comporte un lien vers
# la table des questions. 

function rt_question {
    local q=$1
    shift
    # Used by log2toc.pl
    rt_comment "[cfskitTOC: $q ==" "$@" ":cfskitTOC]"
    rt_section "<a href='#TOC'>" $RT_UP_IMAGE "</a>" "$@" "<a name='$q'></a>"
}

function rt_section {
    rt_start_xml h2
    echo -n "$@"
    rt_stop_xml h2
}

function rt_subsection {
    rt_start_xml h3
    echo -n "$@"
    rt_stop_xml h3
}

function rt_subsubsection {
    rt_start_xml h4
    echo -n "$@"
    rt_stop_xml h4
}

# Cite stdout

function rt_start_stdout {
    rt_start_xml "div class=\"STDOUT\""
}

function rt_stop_stdout {
    rt_stop_xml div
}

# Cite stderr

function rt_start_stderr {
    rt_start_xml "div class=\"STDERR\""
}

function rt_stop_stderr {
    rt_stop_xml div
}

# Emit a link towards a test file $2:

function rt_link_test {
    if [ -x "${RT_LINK_TEST}" ]
    then
        # Let an external script generate the precise link:
        ${RT_LINK_TEST} $1 $2
    else
        echo "$2"
    fi
}

# Transcode HTML special characters.

function rt_transcode_carefully {
    sed -e 's/&/\&amp;/g' -e 's/</\&lt;/g' -e 's/>/\&gt;/g' 
}

# Show the content of a file $2 prefixed with a sentence $1:
# L.Seinturier: now handles correctly <, > and & for HTML.

function rt_echo_file {
    rt_notify_deprecation "[[la commande rt_echo_file est a proscrire, 
utilisez plutot rt_file_echo]]"
    local file=`basename $2`
    echo ; rt_echo "$1" `rt_tagged_embed tt "$file"`
    rt_start_xml "div class=\"FILE\""
    rt_transcode_carefully < "$2" 
    rt_stop_xml div
}

function rt_file_echo {
    local file=$1
    echo
    rt_start_xml "div class=\"FILE\""
    rt_transcode_carefully < "$file"
    rt_stop_xml div
}

# Move (append) a file $2 to another one $3 (use the $1 css class to show 
# the content of $2):

function rt_cat {
    #type="$1"
    #source="$2"
    #target="$3"
    if [ -s "$2" ] 
    then {
        rt_start_xml "div class=\"$1\""
        echo ; rt_transcode_carefully < "$2"
        rt_stop_xml div
        }
    else
        rt_comment "Empty $2"
    fi >> "$3"
}

# Emit a return code $1.

function rt_code {
    if [ "$1" -ne 0 ] 
    then 
        echo "<!-- BadExitCodeWarning Start -->"
        rt_start_xml "div class=\"WARNING\""
        rt_warning "Votre programme s'est termin avec le code d'erreur "
        rt_start_xml "span class=\"CODE\""
        echo -n "$1"
        rt_stop_xml span
        rt_warning ". Voyez le flux d'erreur qui suit pour savoir " \
            "(ventuellement) pourquoi."
        rt_stop_xml div
        echo "<!-- BadExitCodeWarning Stop -->"
    fi
}

# Emit a fact to be recorded in the database. Such a fact may be the
# duration of a program, the computer language used or any other
# numeric or textual information. This information may be used to
# compute some statistics or bonuses or whatever. This may be
# considered as as a generic mechanism to record anything about a tgz.

# CAUTION: Protect your string for inclusion in XML: don't use quotes!

function rt_textual_fact {
    local label="$1"
    local fact="$2"
    rt_start_xml_with_id \
        "FACT question='$RT_Q' label='$label' phrase='$fact' "
    rt_stop_xml FACT
}
function rt_numeric_fact {
    local label="$1"
    local fact="$2"
    rt_start_xml_with_id \
        "FACT question='$RT_Q' label='$label' number='$fact' "
    rt_stop_xml FACT
}

# Emit a mark $1. The mark may be an arithmetic Perl expression.
# NOTE: the mark is truncated with two digits after the decimal dot.

# FUTURE: Add a label after the mark ? Statistics could then be made
# to analyze precisely from where global marks are made of.  FUTURE

function rt_win {
    rt_start_xml "span class=\"MARK\""
    echo 
    rt_start_xml_with_id MARK
    perl -e "printf STDOUT \"%5.2f\", ($*);"
    rt_stop_xml MARK
    rt_stop_xml span
}

# }}}

# Check the existence of a file $1 then it's non emptyness:

function rt_exist {
    rt_echo "Le fichier" `rt_tagged_embed tt "$1"` "existe-t'il ? "
    if [ -f "$1" ]
    then 
         rt_echo "Le fichier" `rt_tagged_embed tt "$1"` "est-il non vide ? "
         if [ -s "$1" ]
         then rt_echo "Bravo, le fichier" `rt_tagged_embed tt "$1"` \
                      "n'est pas vide. "
         else rt_error "Le fichier" `rt_tagged_embed tt "$1"` \
                       "existe mais est vide. "
              exit 2
         fi
    else rt_error "Le fichier" `rt_tagged_embed tt "$1"` "est absent. "
         exit 1
    fi
}

# Check the existence and executability of a file named $1:

function rt_executable_exist {
    rt_echo "Le fichier" `rt_tagged_embed tt "$1"` "existe-t'il ? "
    if [ -f "$1" ]
    then 
         rt_echo "Le fichier" `rt_tagged_embed tt "$1"` "est-il non vide ? "
         if [ -s "$1" ]
         then 
            rt_echo "Le fichier" `rt_tagged_embed tt "$1"` \
                    "est-il ex&eacute;cutable ? "
            if [ -x "$1" ]
            then if cmp -s $1 ${RT_THE_TEST_DIR}/$1
                 then rt_error "Votre fichier" `rt_tagged_embed tt "$1"` \
                            "est le m&ecirc;me que le mien. "
                      exit 4
                 else 
                    rt_echo "Bravo, le fichier" `rt_tagged_embed tt "$1"` \
                            "semble par&eacute; de toutes les qualit&eacute;s. "
                 fi
            else rt_error "Le fichier" `rt_tagged_embed tt "$1"` \
                          "n'est pas ex&eacute;cutable. "
                 exit 3
            fi
         else rt_error "Le fichier"  `rt_tagged_embed tt "$1"` "est vide. "
              exit 2
         fi
    else rt_error "Le fichier" `rt_tagged_embed tt "$1"` "est absent. "
         exit 1
    fi
}

# Compare two files $1 and $2 with diff.

function rt_diff {
    local option='';
    case "X$1" in 
        X-*) options=$1 ; shift ;;
        *)   options=-abBcwi ;;
    esac
    rt_echo "Comparons les fichiers" `basename $1` "et" `basename $2` "..."
    if diff $options "$1" "$2" > ${RT_TMP}/.diff
    then rt_echo "Bravo, je ne vois gu&egrave;re de diff&eacute;rences."
    else rt_echo "Les fichiers ont l'air diff&eacute;rents."
         rt_start_xml "div class=\"STDERR\""
         cat ${RT_TMP}/.diff
         rt_stop_xml div
         exit 1
    fi
    rm -f ${RT_TMP}/.diff
}

# Compare a given prefix (a certain number $3 of lines) of two files
# ($1 and $2).  FUTURE: There are options in diff to nicely report
# changed lines. Whitespaces are not considered.

function rt_prefix_diff {
    local option='';
    case "X$1" in 
        X-*) options=$1 ; shift ;;
        *)   options=-abBcwi ;;
    esac
    rt_echo "Comparons les $3 premi&egrave;res lignes des fichiers" \
        `basename $1` "et" `basename $2` "..."
    head --lines=$3 "$1" > "$1.$3"
    head --lines=$3 "$2" > "$2.$3"
    if diff $options "$1.$3" "$2.$3" > ${RT_TMP}/.diff
    then rt_echo "Bravo, les $3 premi&egrave;res lignes sont comparables."
    else rt_echo "Les fichiers ont l'air diff&eacute;rents."
         rt_start_xml "div class=\"STDERR\""
         cat ${RT_TMP}/.diff
         rt_stop_xml div
         exit 1
    fi
    rm -f ${RT_TMP}/.diff
    rm -f "$1.$3" "$2.$3"
}

# Compare two files ($2, $3), compute the number of lines to delete or
# add or substitute to pass from one to the other (the Levenshtein's
# distance) then compute the fraction of the final mark ($1) [If the
# two files are not equal then the final mark is decreased as if in
# rt_scale.] NOTA: the first file is assumed to be the teacher's one.

# The --ignore-bad-exit-code option may appear as first argument in
# which case the warning that may be produced if a program returns a
# bad exit code is removed from the files to compare (see 

function rt_compare {
    local IGNORE_BAD_EXIT_CODE=''
    if [ "$1" = '--ignore-bad-exit-code' ]
    then IGNORE_BAD_EXIT_CODE=$1
         shift
    fi
    local total=$1
    shift
    local ft=$1
    local fs=$2
    rt_echo "Je mesure la diff&eacute;rence entre les fichiers" \
            `basename $ft` "et" `basename $fs` "."
    if ${RT_VERBOSE}
    then
        echo "<div align='center'>fichiers &agrave; comparer:"
        echo "<table><tr><td valign='top'><!-- $fs -->"
        rt_file_echo $fs
        echo "</td><td valign='top'><!-- $ft -->"
        rt_file_echo $ft
        echo "</td></tr></table></div>"
    fi
    # Ignore spaces, case and empty lines:
    perl ${RT_MARK_DIR}/norm.pl \
        $IGNORE_BAD_EXIT_CODE $ft > ${RT_TMP}/levenshtein.t
    perl ${RT_MARK_DIR}/norm.pl \
        $IGNORE_BAD_EXIT_CODE $fs > ${RT_TMP}/levenshtein.s
    if ${RT_VERBOSE}
    then
        echo "<div align='center'>fichiers normalis&eacute;s"
        echo "<table><tr><td valign='top'><!-- levenshtein.s -->"
        rt_file_echo ${RT_TMP}/levenshtein.s
        echo "</td><td valign='top'><!-- levenshtein.t -->"
        rt_file_echo ${RT_TMP}/levenshtein.t
        echo "</td></tr></table></div>"
    fi
    (
      # Only 15 seconds for the comparison:
      . ${RT_MARK_DIR}/limits.sh limited-single 15
      perl ${RT_MARK_DIR}/dist.pl ${RT_TMP}/levenshtein.t \
                                  ${RT_TMP}/levenshtein.s \
                    > ${RT_TMP}/levenshtein.r
    )
    if [ `wc -c < ${RT_TMP}/levenshtein.r` -gt 0 ] 
    then 
      read szt szs d < ${RT_TMP}/levenshtein.r
      if [ $d = 0 ]
      then rt_echo "Excellent!"
      else 
          rt_echo "Il y a $d lignes &agrave; supprimer/modifier/ajouter " \
                  "pour passer des $szt lignes de l'un aux $szs de l'autre."
      fi
      #rt_win `perl -e "my \\$r = $total - (0.5 * $d)/(0.01 + $szt); \
      #                 print ((\\$r>0)?\\$r:0);" ; ` ;
      rt_win `perl -e " \
               my \\$sz = (($szs>$szt)?(2*$szt):($szs+$szt)); \
               my \\$r = $total * (1 - ($d/\\$sz)); \
               print ((\\$r>0)?\\$r:0); \
" ; ` ;
    else 
       # The comparison was probably killed because it was a CPU-hog:
       rt_error "La comparaison s'est mal pass&eacute;e."
       exit 1
    fi
    if ${RT_CLEANUP}
    then rm ${RT_TMP}/levenshtein.[tsr]
    fi
}

# Compare two files ($3 and $4) with diff, computing a mark equals to
# $1 less penalty ($2) times the number of differing lines. Pay
# attention that diff outputs 4 lines for one differing line. The
# final mark cannot be negative.

function rt_compare_with_diff {
    local total=$1
    local penalty=$2
    shift; shift
    local ft=$1
    local fs=$2
    rt_echo "Je mesure la diff&eacute;rence entre les fichiers" \
            `basename $ft` "et" `basename $fs` " avec diff (et apr&egrave;s " \
            "normalisation des fichiers &agrave; comparer)."
    # Ignore spaces, case and empty lines:
    perl ${RT_MARK_DIR}/norm.pl $ft > ${RT_TMP}/levenshtein.t
    perl ${RT_MARK_DIR}/norm.pl $fs > ${RT_TMP}/levenshtein.s
    diff -abBwi ${RT_TMP}/levenshtein.[ts] > ${RT_TMP}/.diff
    N=`wc -l < ${RT_TMP}/.diff`
    rt_echo "La diff&eacute;rence est de l'ordre de $N lignes."
    rt_win `perl -e "my \\$d = $total - $penalty * $N; \
                     print ((\\$d>0)?\\$d:0);" `
    rm ${RT_TMP}/levenshtein.[st]
    rm ${RT_TMP}/.diff
}

# Check that a given regexp $1 may be found in a file $2:

function rt_grep {
    local option='';
    case "X$1" in 
        X-*) options=$1 ; shift ;;
        *)   options='' ;;
    esac
    if [ -f "$2" ] 
    then rt_echo "Trouve-t'on $1 dans" `basename $2` "?"
         if egrep $options "$1" "$2" > /dev/null
         then rt_echo "Bravo, j'ai bien trouv&eacute; $1 dans" `basename $2` "."
         else rt_error "Je ne trouve pas $1 dans" `basename $2` "."
              exit 3
         fi
    else rt_internal_error "Cannot find $2."
         exit 126
    fi
}

# Check that a given regexp $1 (case insensitive) may be found in a file $2:

function rt_grepi {                                           # DEPRECATED!
    rt_notify_deprecation "Use rt_grep -i ... instead"
    rt_grep -i "$1" "$2"
}

# Try to rebuild a program.

RT_EXE_SUFFIX=exe
function rt_rebuild {                                      # DEPRECATED!
    #rt_notify_deprecation "Use rt_build ... instead"
    # Preserve existing .exe if any:
    if [ -f $1.${RT_EXE_SUFFIX} ] 
    then 
        if cmp -s $1.${RT_EXE_SUFFIX} ${RT_THE_TEST_DIR}/$1.${RT_EXE_SUFFIX}
        then rt_warning "Tiens, votre $1.${RT_EXE_SUFFIX} est le " \
            "m&ecirc;me que le mien ?"
        fi
        rt_echo "Je retire votre fichier $1.${RT_EXE_SUFFIX} pour pouvoir " \
            "le reconstruire. "
        mv $1.${RT_EXE_SUFFIX} RT.$1.${RT_EXE_SUFFIX}
    fi
    # start_rebuild is later used by 11guess:
    touch $RT_THE_LOG_DIR/start_rebuild
    sleep 1
    rt_echo "Je tente de construire $1.${RT_EXE_SUFFIX} avec votre "
    rt_file_echo compil-$1.sh
    rt_echo "Voici ce que j'observe:"
    rt_start_stdout
    # May write files up to 10 megs (useful when compiling):
    rt_confiner --output=10000 bash < compil-$1.sh | rt_tee 10k /dev/null
    rt_stop_stdout
}

# The command "rt_build target script" tries to rebuild a program with a 
# script. This is a more versatile function than the previous one.

function rt_build {
    local target=$1
    local script=$2
    # Preserve existing executable if any:
    if [ -f $target ] 
    then 
        if cmp -s $target ${RT_THE_TEST_DIR}/$target
        then rt_warning "Tiens, votre fichier <tt>$target</tt> est le " \
            "m&ecirc;me que le mien ?"
        fi
        rt_echo "Je retire votre fichier <tt>$target</tt> pour pouvoir " \
            "le reconstruire. "
        mv $target RT.$target
    fi
    # start_rebuild is later used by 11guess:
    touch $RT_THE_LOG_DIR/start_rebuild
    sleep 1
    rt_echo "Je tente de construire <tt>$target</tt> avec votre script " \
        "<tt>$script</tt>. "
    rt_file_echo $script
    rt_echo "Voici ce que j'observe:"
    rt_start_stdout
    # May write files up to 10 megs (useful when compiling):
    rt_confiner --output=10000 bash < $script | rt_tee 10k /dev/null
    rt_stop_stdout
}    

# <Christophe.Gonzales@lip6.fr>
# Determiner si un fichier existe ou celui dont le nom est le plus proche.
# C'est surtout pour compenser les erreurs de frappe. La variable 
# RT_TYPO_THRESHOLD indique le nombre maximal d'erreurs de frappes tolerable.
# Usage: rt_guess_file cible fichiers-possibles ...

export RT_TYPO_THRESHOLD=${RT_TYPO_THRESHOLD:-3}

function rt_guess_file {
    local SCORE=""
    local FILE=''
    local BEST_SCORE=1000000
    local BEST_FILE=""
    local TARGET=$1
    local NORMALIZED_TARGET=`rt_normalize_name "$TARGET" `
    local NORMALIZED_FILE
    local COMPARER="perl ${RT_MARK_DIR}/levenshtein.pl"

    shift
    if [ -f "$TARGET" ]
    then echo "$TARGET"
         true
    else
        for FILE in "$@"
        do
          NORMALIZED_FILE=`rt_normalize_name "$FILE" `
          SCORE=` $COMPARER "$NORMALIZED_TARGET" "$NORMALIZED_FILE" `
          if [ $SCORE -lt $BEST_SCORE ]
          then
              BEST_SCORE=$SCORE
              BEST_FILE=$FILE
          fi
        done
        if [ $BEST_SCORE -le ${RT_TYPO_THRESHOLD} ]
        then
            echo "$BEST_FILE"
            #rt_warning "Je ne trouve pas le fichier <tt>$TARGET</tt> mais " \
            #    "j'ai par contre trouv&eacute; <tt>$FILE</tt> " \
            #    "dont le nom s'approche de ce " \
            #    "qui &eacute;tait demand&eacute;. " \
            #    "Je tente de continuer avec! "
            true
        else
            #rt_error "Je ne suis pas arriv&eacute; &agrave; trouver " \
            #    "un seul fichier dont le nom ressemble &agrave; " \
            #    "<tt>$TARGET</tt> ce qui &eacute;tait pourtant " \
            #    "demand&eacute;. "
            false
        fi
    fi
}

# <Christophe.Gonzales@lip6.fr>
# Cette fonction normalise un nom de fichier et n'est utilisee que par
# la fonction precedente rt_guess_file.

function rt_normalize_name {
    echo "$1" | \
    sed -e 'y/ABCDEFGHIJKLMNOPQRSTUVWXYZ/abcdefghijklmnopqrstuvwxyz/' \
        -e 's/ //g' -e 's/_/-/g' -e 's|^./||'
}

# Essayer de trouver en quel langage est ecrit le programme ?  C'est
# probablement un fichier compile (donc lu) apres la creation de
# start_rebuild.

function rt_guess_language {
    local LANGUAGE=
    if [ -f $RT_THE_LOG_DIR/start_rebuild ]
    then 
        find . -type f -anewer $RT_THE_LOG_DIR/start_rebuild  -a \
            -not -newer $RT_THE_LOG_DIR/start_rebuild \
            -print > $RT_THE_LOG_DIR/fresh
        while read f 
          do
          case "$f" in
              *.c) 
                  LANGUAGE="C $LANGUAGE"
                  ;;
              *.pl|*.pm)
                  LANGUAGE="Perl ${LANGUAGE}"
                  ;;
              *.ad[bs]|*.ali)
                  LANGUAGE="Ada ${LANGUAGE}"
                  ;;
              *.ml*)
                  LANGUAGE="Caml ${LANGUAGE}"
                  ;;
              *.java)
                  LANGUAGE="Java ${LANGUAGE}"
                  ;;
              *.py)
                  LANGUAGE="Python ${LANGUAGE}"
                  ;;
              *.awk)
                  LANGUAGE="Awk ${LANGUAGE}"
                  ;;
              *.sed)
                  LANGUAGE="sed ${LANGUAGE}"
                  ;;
              *.awk)
                  LANGUAGE="awk ${LANGUAGE}"
                  ;;
              *.scm)
                  LANGUAGE="Scheme ${LANGUAGE}"
                  ;;
              */compil*.sh)
                  # On ignore ce fichier
                  ;;
              *.sh)
                  LANGUAGE="sh ${LANGUAGE}"
                  ;;
              *.php)
                  LANGUAGE="php ${LANGUAGE}"
                  ;;
              *.o|*.exe|a.out|*'~'|*.bak)    # On ignore ces fichiers
                  ;;
              *)
              #echo "le fichier $f a aussi &eacute;t&eacute; lu " \
              #    "r&eacute;cemment mais je ne sais pas ce que c'est. "
              ;;
              esac
        done < $RT_THE_LOG_DIR/fresh
        # Ne pas effacer, cela peut encore servir!
        #rm $RT_THE_LOG_DIR/start_rebuild 
        rm $RT_THE_LOG_DIR/fresh
        if [ -n "$LANGUAGE" ] 
        then 
            rt_echo "Je crois que vos programmes sont en $LANGUAGE. "
            echo $LANGUAGE | tr ' ' '\n' | sort -u | \
            while read w 
            do
              rt_textual_fact "language" "$w"
            done
        fi
    fi
}

# rt_rebuild moves any previous .exe before trying to rebuild it. Here
# we restore the old .exe but only if the new one does not exist.

function rt_recuperate {
    if [ -x "$1" ]
    then 
        true
    else
        if [ -x "RT.$1" ]
        then 
            rt_warning "Je n'ai pas r&eacute;ussi &agrave; " \
            "reg&eacute;n&eacute;rer votre $1" \
            "mais j'ai remarqu&eacute; qu'un autre $1" \
            "trainait dans votre r&eacute;pertoire. " \
            "Je continue la suite des tests" \
            "avec cette ancienne version."
            mv "RT.$1" "$1"
        fi
    fi
}

# Transmit a number of bytes in a file. If the number of bytes exceeds
# $1 then echo a warning and discard the rest of the stream. Store the
# stdin in the $2 file as well as output it on stdout (as the usual
# tee).

function rt_tee {
    perl ${RT_MARK_DIR}/tee_head.pl "$1" "$2"
}

# Compute a mark. The student yields s, the expected value was t. If
# t=s then the student wins f. Otherwise he wins f*(1 - |t-s|/(|t|+|s|)).
# Attention: t should not be null, use rt_degress instead!
#
#    ^
# f  |   =
#    |  / \
#    | /   \
#    |/     \_______
#    +---+---+------->
#        t           s

# Use a perl library for all these functions ???                    FUTURE

function rt_scale {                                               # DEPRECATED!
    #f=$1
    #t=$2  # $t != 0 of course
    #s=$3
    perl -e "my \$d = $1 * (1. - (abs($2-$3)/(abs($2)+abs($3)))); \
             print ((\$d>0)?\$d:0);"
}

# Compute a mark: the student yields s, the expected value was 0. If 
# s=0 then the student wins f. Otherwise he wins sup(0,f-s*penalty).
# 
#    ^
# f  =
#    |\
#    | \
#    |  \_______
#    +---+---+------->
#        t           s

function rt_degress {
    #f=$1
    #penalty=$2
    #s=$3
    perl -e "my \$d = $1 - ($2 * $3);
             print ((\$d>0)?\$d:0);"
}

# Get the names of all RT_ variables or rt_ functions.

function rt_get_rt_variables {
    declare | sed -n -e '/^RT_/s/=.*$//p'
}
function rt_get_rt_functions {
    declare -f | sed -n -e '/^rt_/s/=.*$//p'
}
function rt_hide_rt_stuff {
    unset -v `rt_get_rt_variables`
    unset -f `rt_get_rt_functions`
    # These variables may be set via the Makefile:
    unset -v MAKEFLAGS CFSKIT_DIR HERE
}

# Force to return a precise return code.

function rt_exit {
    local code=$1
    # expensive!               FIXME
    export code
    (exit $code)
}

# Confine a computation. It is possible to change the max CPU with the
# --cpu= option. This seems to be safer than the confiner executable
# that has problem to execv an interpreted script.

function rt_confiner_OLD {
    rm -f ${RT_TMP}/.lastcode >/dev/null 2>&1
    (
      # copy this otherwise it will be hidden!
      lc=${RT_TMP}/.lastcode
      . ${RT_MARK_DIR}/limits.sh student
      t=30                  # 30 seconds
      f=200                 # 200 k maximum file size
      while [ $# -gt 0 ]
      do
        case "X$1" in
            X--cpu=*)
                t=$1
                shift
                t=${t#--cpu=}
                ;;
            X--output=*)
                f=$1
                shift
                f=${f#--output=}
                ;;
            *)
                break
                ;;
        esac
      done
      ulimit -t $t 2>/dev/null
      ulimit -f $f 2>/dev/null
      #rt_echo "Running $@ ..."                              # DEBUG
      rt_hide_rt_stuff
      "$@"
      lastcode=$?
      echo $lastcode > ${lc}
      #echo "End of $@ (code is $lastcode)."                 # DEBUG
    ) &
    rt_cmdpid=$!

    t=30                  # 30 seconds
    while [ $# -gt 0 ]
    do
      case "X$1" in
          X--cpu=*)
              t=$1
              shift
              t=${t#--cpu=}
              ;;
          *)
              break
              ;;
      esac
    done

    # Wait for $@ to finish or at least t seconds:
    while [ $t -ge 0 ]
    do 
      #rt_echo "[$t] I'm waiting for $@" ...                         # DEBUG
      if [ -s ${RT_TMP}/.lastcode ]
      then 
          #rt_echo "[$t] last code is `cat ${RT_TMP}/.lastcode`."     # DEBUG
          break
      else
          sleep 1
          t=$(( $t - 1 ))
      fi
    done
    
    # 111 means "killed by timeout"
    if [ ! -s ${RT_TMP}/.lastcode ]
    then 
        #rt_echo "[-1] Killing $@" ...                                # DEBUG
        kill -9 $rt_cmdpid >/dev/null 2>&1
        sleep 1
        if [ ! -s ${RT_TMP}/.lastcode ]
        then 
            #rt_echo "[-2] setting last code to 111."                 # DEBUG
            echo 111 > ${RT_TMP}/.lastcode
        fi
    fi
    
    # return that error code:
    code=`cat ${RT_TMP}/.lastcode`
    if [ $code -gt 0 ]
    then rt_code $code
    fi
    rt_exit $code
}

# New version of rt_confiner

function rt_confiner {
    local CHECK_RETURN_CODE=true

    rm -f ${RT_TMP}/.lastcode >/dev/null 2>&1
    for option in "$@"
    do
      case "X$option" in 
          X--ignore-return-code)
              CHECK_RETURN_CODE=false
              ;;
          *)
              ;;
      esac
    done
    # <Christophe.Gonzales@lip6.fr>: Pour des raisons inconnues, le cat|
    # qui suit semble necessaire !?
    cat | (
      # Start a watchdog
      rt_start_watchdog --watched-pid=`eval 'echo $$'` "$@" </dev/null &

      . ${RT_MARK_DIR}/limits.sh student
      t=30                  # 30 seconds
      f=200                 # 200 k maximum file size
      while [ $# -gt 0 ]
      do
        case "X$1" in
            X--cpu=*)
                t=$1
                t=${t#--cpu=}
                shift
                ;;
            X--watchdog|X--ignore-return-code)
                shift
                ;;
            X--output=*)
                f=$1
                f=${f#--output=}
                shift
                ;;
            *)
                break
                ;;
        esac
      done
      ulimit -t $t 2>/dev/null
      ulimit -f $f 2>/dev/null
      #rt_echo "Running $@ ..."                              # DEBUG
      rt_hide_rt_stuff
      "$@"
    )
    lastcode=$?
    if [ -s ${RT_TMP}/.lastcode ]
    then lastcode=` cat ${RT_TMP}/.lastcode `
    else echo $lastcode > ${RT_TMP}/.lastcode
    fi
    if $CHECK_RETURN_CODE
    then rt_code $lastcode
    fi
}
#NOTE: If one wants to check the return code, just write:
#      rt_code ` cat ${RT_TMP}/.lastcode `
# before this file disappears!

function rt_start_watchdog {
    t=30                  # 30 seconds
    watchdog=false        # terse watchdog
    while [ $# -gt 0 ]
    do
      case "X$1" in
          X--cpu=*)
              t=$1
              t=${t#--cpu=}
              shift
              ;;
          X--watchdog)
              watchdog=true
              shift
              ;;
          X--watched-pid=*)
              pid=$1
              pid=${pid#--watched-pid=}
              shift
              ;;
          X--ignore-return-code)
              shift
              ;;
          *)
              break
              ;;
      esac
    done

    # Wait at most t seconds or until ${RT_TMP}/.lastcode appears:
    while [ $t -ge 0 ]
    do 
      if $watchdog
      then echo "[$t] I'm waiting for ($pid) $@" ... 1>&2
      fi
      if [ -s ${RT_TMP}/.lastcode ]
      then 
          if $watchdog
          then echo "[$t] last code is `cat ${RT_TMP}/.lastcode`." 1>&2
          fi
          break
      else
          sleep 1
          t=$(( $t - 1 ))
      fi
    done
    
    # 111 means "killed by timeout"
    if [ ! -s ${RT_TMP}/.lastcode ]
    then 
        if $watchdog
        then rt_echo "[-1] Starting to kill ($pid) $@" ... 1>&2
        fi
        echo 111 > ${RT_TMP}/.lastcode
        kill -9 $pid >/dev/null 2>&1
    fi
}

# end of runtestlib.sh
