#! /usr/bin/perl
# $Id: dist.pl,v 1.6 2004/04/15 20:30:30 queinnec Exp $
# Copyright (C) 2002 by <Christian.Queinnec@lip6.fr>

use diagnostics;
diagnostics::enable();
use strict;

my $file1;
my $file2;
my $usage = "$0 file1 file2
Computes the Levenshtein distance from file1 to file2 that is, the number
of lines to change, add or delete to make file1 looks like file2.
The output is a single line with three numbers (separated with a space):
    -- the number of lines of the first file,
    -- the number of lines of the second file,
    -- the Levenshtein distance.
";

if ( scalar(@ARGV) == 2 ) {
    ($file1, $file2) = @ARGV;
    die "No such $file1" unless -f $file1;
    die "No such $file2" unless -f $file2;
} else {
    die $usage;
}

my @file1 = split(/\n/, `cat $file1`);
my @file2 = split(/\n/, `cat $file2`);

### FUTURE: normalize the lines of the two files ???

# The array of distances.
#     i \ j 0 1 2 ... scalar(@file2)
#     0     0 1 2
#     1     1 ? ?
#     2     2 ? 
#    ...

my @d = [];
$d[0][0] = 0;

foreach my $i ( 1 .. scalar(@file1) ) {
    $d[$i][0] = $i;
}
foreach my $j ( 1 .. scalar(@file2) ) {
    $d[0][$j] = $j;
}

sub max {
    my ($a, $b) = @_;
    if ( $a > $b ) {
        return $a;
    } else {
        return $b;
    }
}

sub min {
    my ($a, $b) = @_;
    if ( $a > $b ) {
        return $b;
    } else {
        return $a;
    }
}

### FUTURE Compare with other predicates ???

sub compare {
    my ($line1, $line2) = @_;
    return $line1 eq $line2;
}

if (    scalar(@file1) != 0 
     && scalar(@file2) != 0 ) {

    foreach my $diagonal ( 2 .. (scalar(@file1)+scalar(@file2)) ) {
        foreach my $i ( 1 .. $diagonal ) {
            #print STDERR "[diagonal $diagonal]";
            next if ( $i > scalar(@file1) );
            my $j = $diagonal - $i;
            next if ( $j == 0 || $j > scalar(@file2) );
            #print STDERR "($i, $j)";
            my $line1 = $file1[$i-1];
            my $line2 = $file2[$j-1];
            my $ul = $d[max(0, $i-1)][max(0, $j-1)]; # up left
            my $u  = $d[max(0, $i-1)][$j]; # up
            my $l  = $d[$i][max(0, $j-1)]; # left
            #print STDERR "(ul $ul, u $u, l $l)";
            if ( compare($line1, $line2) ) {
                $d[$i][$j] = min($ul, min(1 + $l, 1 + $u));
            } else {
                $d[$i][$j] = min(1 + $ul, min(1 + $l, 1 + $u));
            }
        }
    }
}

print scalar(@file1), " ",
      scalar(@file2), " ",
      $d[scalar(@file1)][scalar(@file2)], "\n";

# end of dist.pl
