// $Id: TestRun.java,v 1.6 2003/08/13 19:04:21 queinnec Exp $

package fr.lip6.qnc.cfskit;

import junit.framework.*;
import junit.textui.*;
import java.lang.reflect.*;
import java.util.Enumeration;
import java.util.Vector;
 
/** 
    This class will build and run the tests from the mentioned
    TestClasses. To run it, just say something like:

    java fr.lip6.qnc.cfskit.TestRun your.class.name ...

    Each your.class.name will be loaded and then run with Junit. An
    easily parsable summary is printed after.

    If your.class.name extends fr.lip6.qnc.cfskit.TestCase then the
    summary will also display the number of assertions run during
    these tests. This is particularly useful for the cfskit where the
    number of assertions is an indicator of the difficulty of the
    associated class the students have to write. 

**/

public class TestRun extends fr.lip6.qnc.cfskit.TestCase {

  /** This method waits for the name of some classes and will collect
      and gather their suite of tests into one suite that will be run.
   **/
  
  public static void main (String[] args) {
    // Share args with getNumberOfAssertions():
    if ( args.length > 0 ) {
      System.out.println(banner);
      TestSuite suite = new TestSuite();
      for ( int i=0 ; i<args.length ; i++ ) {
        collect(suite, args[i]);
      }
      TestResult result = junit.textui.TestRunner.run(suite);
      String msg = "\n[[[" 
        + getNumberOfAssertions() + " Assertions, "
        + result.runCount() + " Tests, " 
        + result.failureCount() + " Failures, " 
        + result.errorCount() + " Errors]]]";
      System.out.println(msg);
    } else {
      System.err.println(usage);
      System.exit(1);
    }
  }

  private final static String banner = 
    TestRun.class.getName() 
    + " $Revision: 1.6 $ with help of the JUnit framework.";
  private final static String usage = 
    "Usage: java " + TestRun.class.getName() + " classNames ...";

  public TestRun (String name) {
    super(name);
  }

  public static int getNumberOfAssertions () {
    int count = 0;
    Enumeration e = testClasses.elements();
    while ( e.hasMoreElements() ) {
      try {
        Class clazz = (Class) e.nextElement();
        Field field = clazz.getField("count");
        count += field.getInt(null);
      } catch (IllegalArgumentException exc) {
        System.err.println(exc.getMessage());
        // Nothing to do.
      } catch (IllegalAccessException exc) {
        System.err.println(exc.getMessage());
        // Nothing to do.
      } catch (NoSuchFieldException exc) {
        System.err.println(exc.getMessage());
        // Nothing to do.
      }
    }
    return count;
  }
  private static Vector /* of Class */ testClasses = new Vector();

  /** Add all the tests present in the classfile to suite. The
      classfile should be the name of a class, an heir to TestCase.
      Its test* methods are then collected to be adjoined to the
      current suite. 
  **/

  public static void collect (TestSuite suite, String classfile) {
    try {
      Class clazz = Class.forName(classfile);
      testClasses.add(clazz);
      suite.addTestSuite(clazz);
    } catch (ClassNotFoundException e) {
      System.err.println("Cannot find or load classfile " + classfile);
      // Stop loading classes ...
      System.err.println(usage);
      System.exit(1);
    }
  }

}

// end of TestRun.java
