#! /usr/bin/perl

# Syntax: handle_plagiats.pl regexp4files.NEAR 
# Adjoin some text to every log file in $RT_LOG_DIR if plagiat.pl 
# recorded some near misses in PLAGIATS.

use diagnostics;
diagnostics::enable();
use strict;
use File::Basename;
use Data::Dumper;

my $files = shift;
my $clipped_length = 60;
my $different_lines_threshold = 50;

# Remove spaces from both ends.

sub trim {
    local ($_) = @_;
    chomp;
    return $1 if /^\s*(.*)\s*$/;
    return $_;
}

# Read $filename and clip all lines to $clipped_length characters.

sub read_and_clip {
    my ($size, $filename) = @_;
    my $content = do { local( @ARGV, $/ ) = $filename; <> };
    $content =~ s/\n(.{0,$size})([^\n]*)\n/\n$1\n/g;
    return $content;
}

print "<html><head><title>Plagiats</title>
</head><body><h1>Plagiats</h1>
";

sub handle {
    my ($file, $csize1, $csize2, $csizes, $other, $delta) = @_;
    my $size1 = ` wc -l < $file `;
    my ($uuid1) = ($file =~ m|/(........-....-....-....-............)/|);
    my $content1 = read_and_clip($clipped_length, $file);
    print STDERR "NearMiss between $file, $uuid1, "; #DEBUG
    my $size2 = ` wc -l < $other `;
    my ($uuid2) = ($other =~ m|/(........-....-....-....-............)/|);
    my $content2 = read_and_clip($clipped_length, $other);
    print STDERR "and $other, $uuid2\n"; #DEBUG
    my $ignore = ` diff -e $file $other > /tmp/t.diff`;
    my $diff = do { local( @ARGV, $/ ) = "/tmp/t.diff"; <> };
    my $sizediff = ` wc -l < /tmp/t.diff `;
    print "
<p> Ces copies <a href='$uuid1/all.html'>A</a>
et <a href='$uuid2/all.html'>B</a> 
sont tr&egrave;s proches (<tt>
lignes du <a href='$uuid1/ALL.JAVA'>fichier1</a>: $size1,
taille fichier1 compress&eacute;: $csize1,
lignes du <a href='$uuid2/ALL.JAVA'>fichier2</a>: $size2,
taille fichier2 compress&eacute;: $csize2,
taille fichiers1et2 compress&eacute;s ensemble: $csizes
delta: $delta%</tt>,
lignes differentes: $sizediff
).</p>
";
    # faire calcul plus fin avec tous ces chiffres ???
    if ( $sizediff < $different_lines_threshold ) {
        print "<div align='center'>
<table border=1><tr><td width='50%' valign='top'><pre> 
$content1
</pre></td><td width='50%' valign='top'><pre> 
$content2
</pre></td></tr></table></div>
";
    };
}

foreach my $file ( glob($files) ) {
    if ( -f "$file.NEAR" ) {
        open(IN1, "< $file.NEAR")
            || die "Cannot read $file.NEAR";
        while ( <IN1> ) {
            if ( /^(\d+) (\d+) (\d+) (.*)\t\[delta=(-?\d+)%\]$/ ) {
                my ($csize1, $csize2, $csizes, $other, $delta) 
                    = ($1, $2, $3, $4, $5);
                handle($file, $csize1, $csize2, $csizes, $other, $delta);
            } else {
                die "Unrecognized line: $_";
            }
        }
        close(IN1);
    }
}

print "<p>et c'est tout!</p></body></html>
";

# end of handle_plagiats.pl
