<?php
// $Id: libcfskit.php,v 1.10 2004/04/19 15:34:18 queinnec Exp $
// A common library of functions to display results of CFS.

setlocale(LC_ALL, "fr_FR.iso885915");

function report_error_and_die ($message) {
     echo "\n\n<div style=\"color: red; font-size: 32px;\">$message</div>\n\n";
     trigger_error($message);
     exit;
}

function generate_common_bar ($module = false) {
     echo "<p><div class=\"commonBar\">"
          . "<a href=\"http://www.upmc.jussieu.fr/\">UPMC</a>&nbsp;&gt;&nbsp;\n"
          . "<a href=\"http://www.ufr-info-p6.jussieu.fr/\">"
          . "UFR informatique</a>&nbsp;&gt;&nbsp;\n"
          . "<a href=\"http://www.infop6.jussieu.fr/licence/\">"
          . "Licence</a>"
          . ($module ? "&nbsp;&gt;&nbsp;$module" : "")
          . "</div></p>\n";
}

// Open a persistent connection to the DB. This function may be used
// more than once (persistent connections are allowed). Use localhost
// instead of papillon since then this script may also run on my home
// box.

function open_connection_to_licence_db ($dbname, $user, $password) {
     $dsn = "host=localhost dbname=$dbname user=$user password=$password";
     $connection = pg_pConnect($dsn);
     if ( ! $connection ) {
          report_error_and_die("Cannot connect to database.\n");
     };
     return $connection;
}

function execute_request ($connection, $query) {
     $request = @pg_Exec($connection, $query);
     if ( ! $request ) {
          report_error_and_die("Cannot select database.\n " 
                               . pg_ErrorMessage($connection));
     };
     return $request;
}

function fetch_one_row ($request, $i = 0) {
     $row = @pg_fetch_array($request, $i);
     if ( $row && is_Array($row) ) {
          return $row;
     } else {
          report_error_and_die("Cannot fetch rows from database.\n" 
                               . pg_ErrorMessage($connection));
     };
}

function number_of_rows ($request) {
     $n = pg_numrows($request);
     if ( $n == -1 ) {
          report_error_and_die("Cannot count rows from database.\n" 
                               . pg_ErrorMessage($connection));
     } else {
          return $n;
     }
}

// Compute the average and standard deviation of an array of numbers.
// These two results are returned packed within an array. array[0] is
// the average while array[1] is the standard deviation.

function & computeStatistics (&$array) {
     $count = 0;
     $sum = 0.0;
     foreach ($array as $item) {
          $count++;
          $sum += $item;
     }
     $average = $sum/$count;
     $variance = 0.0;
     foreach ($array as $item) {
          $s = $item - $average;
          $variance += $s * $s;
     }
     $ecartType = sqrt($variance/$count);
     $results = array($average, $ecartType);
     return $results;
}

class Student {
     var $name;     // name of student that is "nom prenom"
     var $firstname;
     var $lastname;
     var $husbandname;
     var $id;       // studentID
     var $login;    // login of the student
     var $notes;    // a hash table mapping modules names to marks
     var $rank;     // rank among the cohort

     function set_rank ($rank) {
          $this->rank = $rank;
     }
     function add_mark ($module, $mark) {
          // There's a problem of LOCALE between perou and papillon:
          // This hack solves it !!!
          //var_dump($mark); // DEBUG (a string)
          $mark1 = 0.0 + $mark;
          $mark2 = 0.0 + strtr($mark, ".", ",");
          $mark = max($mark1, $mark2);
          //var_dump($mark); // DEBUG (a float now)
          $this->notes[$module] = $mark;
     }
     // return the number of marks
     function marks () {
          return count($this->notes);
     }
     // return the average mark
     function average () {
          $count = 0;
          $total = 0;
          foreach ($this->notes as $module => $mark) {
               $count++;
               $total += $mark;
          }
          if ( $count > 0 ) {
               return ($total/$count);
          } else {
               return "?";
          }
     }
     function Student ($id, $prenom, $nom, $nomMarital) {
          $prenom = strtolower($prenom);
          $this->firstname = $prenom;
          $this->lastname = $nom;
          $this->husbandname = $nomMarital;
          if ( $nomMarital && $nomMarital != '' ) {
               $this->name = "$nom, $prenom, $nomMarital";
          } else {
               $this->name = "$nom, $prenom";
          }
          $this->id = $id;
          $this->notes = array();
          // Don't record in $lesEtudiants !
     }
}

class StudentCFS extends Student {
     var $session;   // The CFS session or DTH leg
     var $final;     // Final mark (replaces $total)

     function total ($lesQuestions) {
          $total = 0.0;
          foreach ($lesQuestions as $question) {
               $mark = $this->notes[$question];
               $total += $mark;
          }
          $total = min(20.0, ceil($total));
          return $total;
     }

     function StudentCFS ($id, $prenom, $nom, $nomMarital, $session) {
          $this->session = $session;
          $this->Student($id, $prenom, $nom, $nomMarital);
     }
}

// Return an array with the names of the questions of the examination.
// Usually something as ['Q1', 'Q2', ... ].

function & get_examination_questions ($db_prefix) {
     global $db_name, $db_user, $db_password;
     $result = array();
     $connection = 
	open_connection_to_licence_db($db_name, $db_user, $db_password);
     $query = "
select distinct cm.question
from ${db_prefix}_mark cm
order by cm.question
";
     $request = execute_request($connection, $query);
     for ( $i=0 ; $i<number_of_rows($request) ; $i++ ) {
          $line = fetch_one_row($request, $i);
          $result[] = $line["question"];
     }
     if ( count($result) > 0 ) {
          return $result;
     } else {
          report_error_and_die("No questions at all within ${db_prefix}_mark");
     }
}

function wrap_within_quotes ($s) {
     return "'" . $s . "'";
}

// Return an hashtable mapping uuid to a StudentCFS object. This
// object describes a student with all its marks, where are his
// deliveries, the associated logs etc.

function & extract_all_students_marks ($db_prefix) {
     global $db_name, $db_user, $db_password, $db_prefix, $db_formations;
     $formations = join(', ', array_map("wrap_within_quotes", $db_formations));
     $lesEtudiants = array();
     $connection = 
	open_connection_to_licence_db($db_name, $db_user, $db_password);
     $query = "
select p.id, p.firstname, p.lastname, p.husbandname, 
       cs.session, cm.question, max(cm.mark) as mark
from ${db_prefix}_student cs, ${db_prefix}_mark cm, person p, login l
where cm.uuid = cs.uuid
  and cs.login = l.login
  and l.valid
  and l.student_id = p.id
  and p.valid
  and l.formation_id in (select f.id from Formation f 
                        where f.nickname in ($formations)
                          and f.valid)
group by cs.login, p.id, p.firstname, p.lastname, p.husbandname, 
         cs.session, cm.question
order by p.lastname, p.firstname, p.husbandname
";
     //echo "<pre>$query</pre>\n";
     $request = execute_request($connection, $query);
     for ( $i=0 ; $i<number_of_rows($request) ; $i++ ) {
          $line = fetch_one_row($request, $i);
          $id = $line["id"];
          $prenom = $line["firstname"];
          $nom = $line["lastname"];
          $nomMarital = $line["husbandname"];
          $session = $line["session"];
          $question = $line["question"];
          $mark = $line["mark"];
          $s =& $lesEtudiants[$id];
          if ( ! $s ) {
               $s =& new StudentCFS($id, $prenom, $nom, $nomMarital, $session);
               $lesEtudiants[$id] =& $s;
          };
          if ( array_key_exists('login', $line) ) {
               $s->login = $line["login"];
          };
          $s->add_mark($question, $mark);
     }
     return $lesEtudiants;
}

// Generate a link to the same page with new query parameters. 

function generate_cfs_details (& $student) {
     global $stats;
     $url = "?id=$student->id&login=$student->login";
     if ( isset($stats) && $stats>0 ) {
          $url .= "&stats=1";
     };
     return "<a href='$url' target='_blank'>$student->name</a>";
}

// This class describes a delivery, the marks for every question.

class Submission {
     var $name;    // name of student that is "nom prenom"
     var $uuid;    // 
     var $tgz;     // 
     var $notes;   // a hash table mapping questions to marks

     function add_mark ($question, $mark) {
          // There's a problem of LOCALE between perou and papillon:
          $mark1 = 0.0 + $mark;
          $mark2 = 0.0 + strtr($mark, ".", ",");
          $mark = max($mark1, $mark2);
          $this->notes[$question] = $mark;
     }

     function Submission ($prenom, $nom, $uuid, $tgz) {
          global $lesSoumissions;
          $this->name = "$nom $prenom";
          $this->uuid = $uuid;
          $this->tgz = $tgz;
          $this->notes = array();
     }
}

// Order tgz by names.

function tgz_before ($a, $b) {
     return strcmp($a->tgz, $b->tgz);
}

// Return an hashtable mapping uuid to Submissions.

function & extract_cfs_submission ($id, $lesQuestions) {
     global $db_name, $db_user, $db_password, $db_prefix, $db_formations;
     $formations = join(', ', array_map("wrap_within_quotes", $db_formations));
     $lesSoumissions = array();
     $connection = 
	open_connection_to_licence_db($db_name, $db_user, $db_password);
     $query = "
select l.login, p.firstname, p.lastname, p.husbandname,
       cm.uuid, cs.tgz, cm.question, cm.mark
from person p, login l, ${db_prefix}_student cs, ${db_prefix}_mark cm
where p.id = $id
  and p.valid
  and l.student_id = p.id
  and l.valid
  and cs.login = l.login
  and l.formation_id in (select f.id from Formation f 
                        where f.nickname in ($formations)
                          and f.valid)
  and cm.uuid = cs.uuid
order by p.lastname, p.firstname,cm.uuid;
";
     //echo "<pre>$query</pre>\n";
     $request = execute_request($connection, $query);
     for ( $i=0 ; $i<number_of_rows($request) ; $i++ ) {
          $line = fetch_one_row($request, $i);
          $prenom = $line["firstname"];
          $nom = $line["lastname"];
          $uuid = $line["uuid"];
          $tgz = $line["tgz"];
          $question = $line["question"];
          $mark = $line["mark"];
          $ss =& $lesSoumissions[$uuid];
          if ( ! $ss ) {
               $ss =& new Submission($prenom, $nom, $uuid, $tgz);
               $lesSoumissions[$uuid] =& $ss;
          };
          $ss->add_mark($question, $mark);
     }
     return $lesSoumissions;
}

// TGZ have too long name: shorten them to the last 25 characters:

function shorten_tgz ($tgz) {
     return "..." . substr($tgz, -25);
}

function student_alpha_order ($a, $b) {
     return strcmp($a->name, $b->name);
}


?>
