#! /usr/bin/perl
# Copyright (C) 2001-2002 by <Christian.Queinnec@lip6.fr>
# GPL >= 2

# Serveur de (re-)notations. Il affiche /tmp/logs/index.html, de voir
# les copies annotees et permet de lancer la notation des copies en
# cliquant sur un bouton. Ce serveur ne doit pas rester ouvert!
# Ce code est repris de ~/Cours/Retic/server.pl

use diagnostics;
diagnostics::enable();
use strict;
use IO::Socket;
use POSIX "sys_wait_h";

# Lire man IO::Socket, IO::Socket::INET, Socket, perlfunc, perlipc.

my $port = 56001;
my $verbose = 0;
my $safe = 1;
my $usage = "$0 [-v] [--unsafe] [port]
Lance un serveur sur le port indique ($port par defaut) pour
renoter des copies. Le serveur n'est accessible que de la machine
sur laquelle il tourne (127.0.0.1) sauf si l'option --unsafe apparait.
";

# Analyse les arguments 

foreach ( @ARGV ) {
    if ( /^\d+$/ ) {
        $port = $_;
    } elsif ( /^-(v+)$/ ) {
        $verbose += length($1);
    } elsif ( /^--unsafe$/ ) {
        $safe = 0;
    } else {
        die $usage;
    }
}

# Cration et initialisation d'un serveur pour couter une connexion TCP.

sub initialize_server {
    my $server = IO::Socket::INET->new(LocalPort => $port,
                                       Type => SOCK_STREAM,
                                       Listen => 1)
        || die "Cannot initialize server: $!";
    return $server;
}

# Une mini-fonction implantant une variante extrement simplifiee
# d'HTTP/1.0. Cette variante n'est pas tres securitaire.

sub http {
    my ($client) = @_;
    my $firstLine = <$client>;
    if ( $firstLine =~ /^(GET|POST)\s+
                         ((http:\/\/[^\/]+)?(\/.*?)(\?([^\s]*))?)(\s+
                          (HTTP\/[\.\d]+)?)\s*$/x ) {
        my($method, $url, $path, $query, $protocol_version) =
            ($1, $2, $4, $6, $8);
        if ( $method eq 'POST' ) {
            if ( defined($query) && $query =~ /uuid=(.*)&login=(.*)/ ) {
                my ($uuid, $login) = ($1, $2);
                my $command = "export RT_UUID='$uuid' && "
                    . "make TGZNAME='$login' re.mark.one.tgz ";
                print STDOUT "----------------------------------------\n";
                system($command) &&
                    die "Problems with $command: $!";
                print STDOUT "----------------------------------------\n";
                # handle path as if for a GET request
            } elsif ( defined($query) && $query =~ /all=yes/ ) {
                my $command = "make re.mark.all.tgz ";
                print STDOUT "----------------------------------------\n";
                system($command) &&
                    die "Problems with $command: $!";
                print STDOUT "----------------------------------------\n";
                # handle path as if for a GET request
            } else {
                print STDERR "Server [$$] rejects $url\n" if $verbose > 2;
                print $client "HTTP/1.0 404 Missing page\r\n\r\n";
            }
        };
        if ( defined($path) && -f $path ) {
            print STDERR "Server [$$] serving $path ...\n" if $verbose > 2;
            print $client "HTTP/1.0 200 OK\r\n";
            print $client "Server: re-mark-server.pl/1.01\r\n";
            if ( $path =~ /\.html$/ ) {
                print $client "Content-Type: text/html\r\n";
            } elsif ( $path =~ /\.gif$/ ) {
                print $client "Content-Type: image/gif\r\n";
            } elsif ( $path =~ /\.php$/ ) {
                # FUTURE: dup() serait mieux <<<<<<<<<<<<<<<<<<<<<--------
                my $tmp = "tmp$$";
                system "php '$path' > $tmp"
                    || die "Cannot run php";
                print $client `cat < $tmp`;
                unlink $tmp;
                exit(0);
            } else {
                print $client "Content-Type: text/plain\r\n";
            }
            print $client "Content-Length: ", (-s $path), "\r\n";
            print $client "\r\n";
            print $client `cat < $path`;
        } elsif ( defined($path) && -d $path ) {
            print STDERR "Server [$$] serving $path/ ...\n" if $verbose > 2;
            print $client "HTTP/1.0 200 OK\r\n";
            print $client "Content-Type: text/html\r\n";
            print $client "\r\n";
            print $client "<html><head><title> Directory $path</title>\n";
            print $client "</head><body><h2>$path</h2><p><tt><ul>\n";
            opendir(DIR, $path)
                || die "Cannot read $path";
            my @files = sort(readdir(DIR));
            foreach my $file (@files) {
                if ( -d "$path/$file" ) {
                    $file = "$file/";
                };
                print $client "<li><a href='$file'>$file</a> (", 
                (-s "$path/$file"), " octets)\n";
            }
            print $client "</ul></tt></p></body></html>\n";
            closedir(DIR);
        } else {
            print STDERR "Server [$$] rejects $url\n" if $verbose > 2;
            print $client "HTTP/1.0 404 Missing page\r\n\r\n";
        }
    } else {
        print STDERR "Server [$$] ignores $firstLine" if $verbose > 2;
        print $client "HTTP/1.0 406 Not understood\r\n\r\n";
    }
}

# Une fonction capturant le signal prvenant du dcs d'un processus
# fils. Cela permet de se dbarrasser des zombies d'Unix. (Voir man
# perlipc) ou the ``camel book'' p340.

sub reaper {
    $SIG{'CHLD'} = \&reaper;
    local $!;
    while ( my $kid = waitpid(-1, WNOHANG) ) {
        if ( $kid > 0 ) {
            print STDERR "Reaped $kid with exit code $?.\n" if $verbose > 2;
        } else {
            last;
        }
    }
}

# Et on place cette fonction pour intercepter le bon signal:

$SIG{'CHLD'} = \&reaper;

# On cre le serveur:

my $server = initialize_server();

# et on le lance. Il n'y a pas moyen de l'arrter. Chaque connexion
# entrante est accepte et le serveur cre un nouveau processus pour
# traiter cette nouvelle connexion.

while ( 1 ) {
    print STDERR "Main server [$$] listening on port $port.\n" if $verbose;
    my $client = $server->accept();
    next unless $client;
    print STDERR "Incoming request from ", $client->peerhost(), 
        ":", $client->peerport(), "\n" if $verbose > 1;
    if ( $safe ) {
        die "Unauthorized access." 
            unless ( $client->peerhost =~ '127.0.0.1');
    };
    my $pid = fork();
    if ( $pid == 0 ) {
        print STDERR "Client handled by Server [$$]\n" if $verbose > 1;
        $server->close();
        http($client);
        print STDERR "Server [$$] exiting.\n" if $verbose > 1;
        exit(0);
    } else {
        $client->close();
    }
}

# fin de re-mark-server.pl
