#! /bin/sh
# $Id: runalltests.sh,v 1.23 2004/05/14 17:47:32 queinnec Exp $
# Copyright (C) 2002 by <Christian.Queinnec@Lip6.fr>

# Usage: /..../runalltests.sh file.tar.gz
# Test the files of the current directory against all the tests associated to
# all questions. This iterates over runtests.sh.

export RT_TGZ=`basename $1`

# Generate a UUID to distinguish that test from any other concurrent tests.

export RT_UUID=${RT_UUID:-`uuidgen`}

# Assume the current directory to hold the files to test.

export RT_PWD=${RT_PWD:-`pwd`}

# Determine the directory where the current script is. This is needed
# to find the library functions.

cd `dirname $0` 
export RT_MARK_DIR=${RT_MARK_DIR:-`pwd`}
cd ${RT_PWD}

# Load a library of utility functions:

. ${RT_MARK_DIR}/runtestlib.sh

# Determine the directory where the tests may be reached (or compute
# it with the directory of the current script). The tests are gathered
# in subdirectories of the RT_TESTS_DIR directory.

export RT_TESTS_DIR=${RT_TESTS_DIR:-${RT_MARK_DIR}}

# Initialize the log files. The .log is the global log, every question
# produces a log, they are all concatenated in the global log.

export RT_LOG_DIR=${RT_LOG_DIR:-${RT_TESTS_DIR}/logs}
if [ -d ${RT_LOG_DIR} ]
then true
else
    echo "Missing directory ${RT_LOG_DIR}/" 1>&2
    exit 1
fi
if [ -d  ${RT_LOG_DIR}/${RT_UUID} ]
then 
    echo "Overwriting directory ${RT_LOG_DIR}/${RT_UUID}" 1>&2
else
    mkdir ${RT_LOG_DIR}/${RT_UUID}
fi
export RT_ALL_LOG=${RT_LOG_DIR}/${RT_UUID}/all.log

# A prefix for temporary stuff.
export RT_TMP=/tmp/${RT_UUID}/tmp

if [ -d ${RT_TMP} ] 
then true
else
    mkdir -p ${RT_TMP}
fi

# Prepare the introduction to the log:
( echo '<!-- CFSKIT versions ' 
  ( cat ${RT_MARK_DIR}/* | ident | grep Id ) 2>/dev/null
  echo '-->'
  # A comment to make easier for external programs to bind the tgz to uuid:
  echo '<!-- DELBUL: ' ${RT_TGZ} ' == ' ${RT_UUID} ' -->'
  echo "
<h1> Copie annot&eacute;e <tt>${RT_TGZ}</tt></h1>
<p> 
Ce document d&eacute;taille la notation de la copie  <tt>${RT_UUID}</tt>  
correspondant &agrave; la soumission <tt>${RT_TGZ}</tt>.
</p><p>
Les questions sont ordonn&eacute;es, les tests dans les
questions sont nomm&eacute;s, des phrases indiquent ce qui se passe, les
entr&eacute;es et les sorties des commandes apparaissent &eacute;galement. Les
points acquis (ou perdus parfois) sont marqu&eacute;s sur fond bleu. Les
temps sont parfois indiqu&eacute;s bord&eacute;s de vert.  Les anomalies sont
&eacute;crites en rouge (notamment le contenu de stderr), 
les &eacute;l&eacute;ments suspects en orange. 
</p><p> 
Correction d&eacute;but&eacute;e le `date`. 
</p>
"
) 1>${RT_ALL_LOG}

# Show DeliveryBuilder diagnostic if present:
if [ -n "`ls diagnostic_livraison_*.html 2>/dev/null`" ]
then {
    echo "<p> Le diagnostic de DeliveryBuilder est: " ;
    perl ${RT_MARK_DIR}/html-extract.pl diagnostic_livraison_*.html ;
    } 1>>${RT_ALL_LOG}
else { 
    echo "<p> Le contenu de votre r&eacute;pertoire est: <pre>" ;
    ls -Rl ;
    echo "</pre></p>" ;
    } 1>>${RT_ALL_LOG}
fi

# Ce marqueur sera remplace par log2html.pl
echo "
<!-- cfskitTOC: here will appear the table of sections :COTtiksfc -->
" 1>>${RT_ALL_LOG}

# On collecte les notes partielles:
mark=0

# Iterate over all questions.

for q in `cd ${RT_TESTS_DIR} ; ls`
do 
    case "$q" in
    *-Q*|Q*)
        if [ -d ${RT_TESTS_DIR}/$q ]
        then
            export RT_LOG=${RT_LOG_DIR}/${RT_UUID}/$q.log
            ${RT_MARK_DIR}/runtests.sh $q
            cat ${RT_LOG} >> ${RT_ALL_LOG}
            qmark=`perl ${RT_MARK_DIR}/cull_marks.pl ${RT_UUID} ${RT_LOG}`
            mark=`perl -e "print ($mark + $qmark);" `
            # clean up partial logs:
            rm ${RT_LOG}
            [ -f ${RT_LOG}.lastcode ] && rm ${RT_LOG}.lastcode
        else
            echo "J'ignore $q." 1>&2
        fi
        ;;
     *) echo "J'ignore ce qu'est $q." 1>&2
        ;; 
    esac
done

# Now we compute the final mark for all questions. We append it to the 
# log file and we also emit it on stderr.

echo "Total obtenu: ${mark}" 1>&2
{ echo -n "<h2> Conclusions </h2>
<p> Finalement, vous avez obtenu <span class=\"MARK_SUM\">"
  rt_start_xml_with_id MARK_SUM_TOTAL
  echo -n "$mark"
  rt_stop_xml MARK_SUM_TOTAL
  echo "</span> point(s) en tout."
  if ${RT_DISPLAY_DISCLAIMER:-true}
  then echo "<br>
Cette somme ne tient pas compte des &eacute;ventuels points de bonification qui 
ne pourraient &ecirc;tre calcul&eacute;s que sur l'ensemble de tous les 
programmes. "
  fi
  echo "</p><hr><p>Correction achev&eacute;e le `date`. </p>"
} 1>>${RT_ALL_LOG} 

# Finalize html log

if ${RT_MAKE_HTML}
then 
    # Compute the table of questions
    perl ${RT_MARK_DIR}/log2toc.pl ${RT_ALL_LOG} \
        > ${RT_LOG_DIR}/${RT_UUID}/all.toc
    # Massage log into html (and insert the table of questions)
    perl ${RT_MARK_DIR}/log2html.pl ${RT_UUID} \
        ${RT_LOG_DIR}/${RT_UUID}/all.toc \
        ${RT_ALL_LOG} \
        > ${RT_LOG_DIR}/${RT_UUID}/all.html
fi

# Erase the tmp directory

if ${RT_KEEP_TMP_DIR}
then false
else rm -rf ${RT_TMP}/
fi

# end of runalltests.sh
