#! /bin/sh
# $Id: runtests.sh,v 1.18 2003/08/13 15:38:16 queinnec Exp $
# Copyright (C) 2002 by <Christian.Queinnec@Lip6.fr>

# Usage: /..../runtests.sh questionI
# Tests appropriate for questionI are in the questionI subdirectory of
# the directory holding the runtests.sh script. All the tests are run in 
# alphabetical order.

# Determine the question to test:

if [ $# -ge 1 ]
then
    export RT_Q=$1
    if [ $# -gt 1 ]
    then 
        echo "Too much parameters: " "$@"
        exit 1
    fi
else
    echo "Missing question identifier"
    exit 1
fi

# Generate a UUID to distinguish that test from any other concurrent tests.

export RT_UUID=${RT_UUID:-`uuidgen`}

# Assume the current directory to hold the files to test.

export RT_PWD=${RT_PWD:-`pwd`}

# Determine the directory where the current script is. This is needed
# to find the library functions.

cd `dirname $0` 
export RT_MARK_DIR=${RT_MARK_DIR:-`pwd`}
cd ${RT_PWD}

# Load a library of utility functions:

. ${RT_MARK_DIR}/runtestlib.sh

# Determine the directory where the tests may be reached (or compute
# it with the directory of the current script). The tests are gathered
# in subdirectories of the RT_TESTS_DIR directory.

export RT_TESTS_DIR=${RT_TESTS_DIR:-${RT_MARK_DIR}}

# Check that the tests specific to the question exists:

if [ -d "${RT_TESTS_DIR}/${RT_Q}" ]
then 
    true
else
    echo "Missing directory ${RT_TESTS_DIR}/${RT_Q}" 
    exit 2
fi

# Export the directory that holds the test scripts:

export RT_THE_TEST_DIR=${RT_TESTS_DIR}/${RT_Q}

# Initialize the log files. The .log is the global log, every test produces
# two partial logs (.out and .err) that are appended (within appropriate
# xml tags) to the global log file.

RT_LOG_DIR=${RT_LOG_DIR:-${RT_TESTS_DIR}/logs}
if [ -d "${RT_LOG_DIR}" ] 
then true
else mkdir ${RT_LOG_DIR}
fi
if [ -d ${RT_LOG_DIR} ]
then true
else echo "Cannot create ${RT_LOG_DIR}/" 1>&2
     exit 1
fi
if [ -d ${RT_LOG_DIR}/${RT_UUID} ]
then true
else mkdir ${RT_LOG_DIR}/${RT_UUID}
fi
export RT_LOG=${RT_LOG:-${RT_LOG_DIR}/${RT_UUID}/${RT_Q}.log}

# The precise log dir for that marking process:
export RT_THE_LOG_DIR=${RT_LOG_DIR}/${RT_UUID}

# A prefix for temporary stuff.
export RT_TMP=${RT_TMP:-/tmp/${RT_UUID}/tmp}

rt_echo "<!-- -->"  1> ${RT_LOG}
if [ -f ${RT_THE_TEST_DIR}/TITLE ] 
then 
    rt_question "${RT_Q}" `cat ${RT_THE_TEST_DIR}/TITLE`
else 
    rt_question "${RT_Q}" "Question ${RT_Q}"
fi 1>> ${RT_LOG}

if [ -f ${RT_THE_TEST_DIR}/PROLOGUE ] 
then 
    cat ${RT_THE_TEST_DIR}/PROLOGUE
fi 1>> ${RT_LOG}

# Perform all the executable tests (with a correct name ie a name
# starting with a two-figures number; skip backups and other files).

for t in `cd ${RT_THE_TEST_DIR} && ls`
do 
    if [ -x ${RT_THE_TEST_DIR}/$t ]
    then case "X$t" in
            *~) true 
                ;;
            X[0-9][0-9]*)
                title=$t
                if egrep -q '^#TIT[RL]E' < ${RT_THE_TEST_DIR}/$t
                then title=`sed -n -e 's/^#TIT[RL]E//p' < ${RT_THE_TEST_DIR}/$t`
                fi
                rt_subsection `rt_link_test ${RT_Q} "$title"`
                (
                . ${RT_MARK_DIR}/limits.sh single
                ${RT_THE_TEST_DIR}/$t 2>> ${RT_TMP}.err || rt_code $? 
                )
                rt_cat STDERR ${RT_TMP}.err ${RT_LOG} && rm ${RT_TMP}.err
                ;;
            *)  true
                ;;
         esac
    else 
         echo "J'ignore le fichier $t" 1>&2 
    fi
done 0< /dev/null   1>> ${RT_LOG}

# Internal function used by run*.sh

function rt_mark_sum {
    echo "<h3> Conclusions question $2 </h3> 
<p> Vous avez donc obtenu "
    rt_start_xml "span class=\"MARK_SUM\""
    rt_start_xml_with_id "MARK_SUM question=\"$2\""
    echo -n "$1"
    rt_stop_xml MARK_SUM
    rt_stop_xml span
    echo "point(s) &agrave; cette question. </p>"
    if [ -f ${RT_THE_TEST_DIR}/EPILOGUE ] 
    then 
        cat ${RT_THE_TEST_DIR}/EPILOGUE
    fi
#    echo "<p>
#Cette somme ne tient pas compte des &eacute;ventuels points de bonification 
#qui ne pourraient &ecirc;tre calcul&eacute;s que sur l'ensemble de tous 
#les programmes fournis. </p>"
}

# Now we compute the final mark for that question. We append it to the 
# log file and we also emit it on stderr.
MARK=`perl ${RT_MARK_DIR}/cull_marks.pl ${RT_UUID} ${RT_LOG}`

rt_mark_sum $MARK $RT_Q      1>> ${RT_LOG}

echo "Somme des points de la question ${RT_Q}: ${MARK}." 1>&2

# end of runtests.sh
